@@ -23,17 +23,17 @@ make_index<-function(letter,number){
 collapse_results<-function(output, collapse){
 ret<-list()
 for(i in 1:length(output)){
-out<-output[[i]]
-num_param<-length(dim(out))-1
-help<-make_index("i",num_param)
-erg<-array(NA,dim=dim(out)[1:num_param])
-loop<-paste(paste(paste("for(i", 1:num_param, " in 1:", dim(out)[1:num_param], "){", sep=""), collapse=""), 
+  out<-output[[i]]
+  num_param<-length(dim(out))-1
+  help<-make_index("i",num_param)
+  erg<-array(NA,dim=dim(out)[1:num_param])
+  loop<-paste(paste(paste("for(i", 1:num_param, " in 1:", dim(out)[1:num_param], "){", sep=""), collapse=""), 
             paste(c("erg[", help, "]<-"), collapse=""),
             paste(c(collapse[[i]], "(na.omit(out[",help,",]))"), collapse=""),
             paste(rep("}", num_param), collapse=""), collapse="")
-eval(parse(text=loop))
-dimnames(erg)<-dimnames(out)[1:num_param]
-ret[[i]]<-erg
+  eval(parse(text=loop))
+  dimnames(erg)<-dimnames(out)[1:num_param]
+  ret[[i]]<-erg
 }
 names(ret)<-names(output)
 ret
@@ -67,14 +67,15 @@ ret
 #' in one of the vectors \code{rows} or \code{cols} (see examples).
 #' 
 #' @param output List of class MonteCarlo generated by \code{MonteCarlo}.
-#' @param rows Vector of parameter names to be stacked in the rows of the table. Ordered from inner to outer.
-#' @param cols Vector of parameter names to be stacked in the cols of the table. Ordered from inner to outer.
+#' @param rows Vector of parameter names to be stacked in the rows of the table. Ordered from the inside to the outside.
+#' @param cols Vector of parameter names to be stacked in the cols of the table. Ordered from the inside to the outside.
 #' @param digits Maximal number of digits displayed in table. Default is \code{digits=4}.
-#' @param collapse Optional list of the same length as \code{output} giving the names of functions to be applied to the 
-#' repective components of \code{output} when collapsing the results to a table. 
-#' By default means are taken. Another example could be \code{sd()}.
+#' @param collapse Optional list of the same length as the list returned by the function *func* supplied to \code{MonteCarlo()}.
+#' This list specifies the names of functions to be applied to the repective components of \code{output} when collapsing the results to a table. 
+#' By default means are taken. Another example could be \code{sd()}. Currently, functions supplied have to return a scalar.
 #' @param transform Optional argument to transform the output table (for example from MSE to RMSE). If a function is supplied
-#' it is applied to all tables. Alternatively, a list of functions can be supplied that has the same length as \code{output}. 
+#' it is applied to all tables. Alternatively, a list of functions can be supplied that has the same length as the list
+#' returned by the function *func* supplied to \code{MonteCarlo()}. 
 #' For tables that are supposed to stay unchanged set list element to \code{NULL}.
 #' @param include_meta Boolean that determines whether the meta data provided by \code{summary()} is included in comments below the table.
 #' Default is \code{include_meta==TRUE}.
@@ -167,14 +168,20 @@ MakeTable<-function(output, rows, cols, digits=4, collapse=NULL, transform=NULL,
   
   if(class(output)!="MonteCarlo")stop("output has to be an object of class MonteCarlo.")
   
+  if(is.null(collapse)==FALSE){
+    for(i in 1:length(collapse)){
+      if(length(do.call(collapse[[i]], list(seq(1,10,1))))>1)stop("Functions included in collapse have to return a scalar.")
+    }
+  }
+  
   full_output_object<-output
   param_list<-output$param_list
   output<-output$results
 
   if(is.null(partial_grid)==FALSE){
     sel<-partial_grid
-    param_list_new<-list()
-    sel_dims<-which(names(sel)%in%names(param_list))
+    #param_list_new<-list()
+    sel_dims<-which(names(param_list)%in%names(sel))
     for(i in 1:length(sel_dims)){
       param_list[[sel_dims[i]]]<-param_list[[sel_dims[i]]][sel[[i]]]
     }
@@ -185,6 +192,8 @@ MakeTable<-function(output, rows, cols, digits=4, collapse=NULL, transform=NULL,
   }
   
   if(any(rows%in%cols))stop("Names in rows and cols must be unique.")
+  helpnames<-c(rows,cols)%in%c(names(param_list),"list")
+  if(any(helpnames==FALSE))stop(paste("The parameter", c(rows,cols)[which(helpnames==FALSE)],"is not part of param_list."))
   
   if(is.null(collapse)){
     collapse<-list()
@@ -564,7 +573,13 @@ MakeTable<-function(output, rows, cols, digits=4, collapse=NULL, transform=NULL,
   cols<-gsub(pattern="_", replacement=paste("\\\\","_", sep=""), cols)
  
   ###--------------------   Loops to print the table from its elements using cat -------------------###
-
+  
+  for(i in 1:length(heads)){
+    while(heads[[i]][length(heads[[i]])]==""){
+      heads[[i]]<-heads[[i]][1:(length(heads[[i]])-2)]
+    }
+  }
+  
   for(i in 1:length(preamble)){cat(preamble[i])}
   if(length(cols)>1){
     for(i in 1:(length(heads)-1)){
